@@ -5,7 +5,8 @@ from tornado.ioloop import IOLoop
 
 from .callbacks_mixin import AppCallBacksMixin
 from .connection import Connection
-from vals import logger
+from .utils import safe_call
+from .vals import logger
 
 
 class Kola(TCPServer, AppCallBacksMixin):
@@ -30,6 +31,7 @@ class Kola(TCPServer, AppCallBacksMixin):
         self.conn_class(self, stream, address)
 
     def run(self, host, port):
+        self._start_repeat_timers()
         self.listen(port, host)
         IOLoop.instance().start()
 
@@ -44,3 +46,14 @@ class Kola(TCPServer, AppCallBacksMixin):
             logger.error("Exception in callback %r", callback, exc_info=True)
 
         IOLoop.instance().handle_callback_exception = exc_handler
+
+    def _start_repeat_timers(self):
+        """
+        把那些repeat timers启动
+        """
+        for func in self.repeat_timer_func_list:
+            safe_call(func)
+        # 执行blueprints
+        for name, bp in self.blueprints.items():
+            for func in bp.repeat_app_timer_func_list:
+                safe_call(func)
