@@ -3,9 +3,10 @@
 import functools
 import time
 from tornado.ioloop import IOLoop
+from utils import safe_call
 
 
-class CallBacksMixin(object):
+class RoutesMixin(object):
     """
     专门做路由管理
     """
@@ -22,8 +23,8 @@ class CallBacksMixin(object):
         if endpoint in self.rule_map and view_func != self.rule_map[endpoint]:
             raise Exception, 'repeat view_func for endpoint: %(endpoint)s, old_view_func:%(old_view_func)s, new_view_func: %(new_view_func)s' % dict(
                 endpoint=endpoint,
-                old_view_func=self.rule_map[endpoint].__name__,
-                new_view_func=view_func.__name__,
+                old_view_func=self.rule_map[endpoint],
+                new_view_func=view_func,
                 )
 
         self.rule_map[endpoint] = view_func
@@ -38,38 +39,25 @@ class CallBacksMixin(object):
     def get_route_view_func(self, endpoint):
         return self.rule_map.get(endpoint)
 
-    def _repeat_timer(self, interval):
-        """
-        每隔一段时间执行(秒)
-        """
-        def inner_repeat_timer(func):
-            @functools.wraps(func)
-            def func_wrapper(*args, **kwargs):
-                # 每次也要再加入
-                IOLoop.instance().add_timeout(time.time() + interval, func_wrapper)
 
-                return func(*args, **kwargs)
-
-            IOLoop.instance().add_timeout(time.time() + interval, func_wrapper)
-            return func_wrapper
-        return inner_repeat_timer
-
-
-class AppCallBacksMixin(CallBacksMixin):
+class AppCallBacksMixin(RoutesMixin):
     create_conn_func_list = None
     before_first_request_func_list = None
     before_request_func_list = None
     after_request_func_list = None
+    before_response_func_list = None
     close_conn_func_list = None
+    repeat_timer_func_list = None
 
     def __init__(self):
         super(AppCallBacksMixin, self).__init__()
-        self.repeat_timer = self._repeat_timer
         self.create_conn_func_list = []
         self.before_first_request_func_list = []
         self.before_request_func_list = []
         self.after_request_func_list = []
+        self.before_response_func_list = []
         self.close_conn_func_list = []
+        self.repeat_timer_func_list = []
 
     def create_conn(self, f):
         """
@@ -103,6 +91,14 @@ class AppCallBacksMixin(CallBacksMixin):
         self.after_request_func_list.append(f)
         return f
 
+    def before_response(self, f):
+        """
+        在 stream.write 之前，传入encode之后的data
+        f(response)
+        """
+        self.before_response_func_list.append(f)
+        return f
+
     def close_conn(self, f):
         """
         连接close之后
@@ -111,26 +107,47 @@ class AppCallBacksMixin(CallBacksMixin):
         self.close_conn_func_list.append(f)
         return f
 
+    def repeat_timer(self, interval):
+        """
+        每隔一段时间执行(秒)
+        """
+        def inner_repeat_timer(func):
+            @functools.wraps(func)
+            def func_wrapper(*args, **kwargs):
+                # 每次也要再加入
 
-class BlueprintCallBacksMixin(CallBacksMixin):
+                result = safe_call(func, *args, **kwargs)
+                IOLoop.instance().add_timeout(time.time() + interval, func_wrapper)
+
+                return result
+
+            self.repeat_timer_func_list.append(func_wrapper)
+            return func_wrapper
+        return inner_repeat_timer
+
+
+class BlueprintCallBacksMixin(RoutesMixin):
     before_request_func_list = None
     after_request_func_list = None
     create_app_conn_func_list = None
     before_app_first_request_func_list = None
     before_app_request_func_list = None
     after_app_request_func_list = None
+    before_app_response_func_list = None
     close_app_conn_func_list = None
+    repeat_app_timer_func_list = None
 
     def __init__(self):
         super(BlueprintCallBacksMixin, self).__init__()
-        self.repeat_app_timer = self._repeat_timer
         self.before_request_func_list = []
         self.after_request_func_list = []
         self.create_app_conn_func_list = []
         self.before_app_first_request_func_list = []
         self.before_app_request_func_list = []
         self.after_app_request_func_list = []
+        self.before_app_response_func_list = []
         self.close_app_conn_func_list = []
+        self.repeat_app_timer_func_list = []
 
     def before_request(self, f):
         """
@@ -180,6 +197,14 @@ class BlueprintCallBacksMixin(CallBacksMixin):
         self.after_app_request_func_list.append(f)
         return f
 
+    def before_app_response(self, f):
+        """
+        在 stream.write 之前，传入encode之后的data
+        f(response)
+        """
+        self.before_app_response_func_list.append(f)
+        return f
+
     def close_app_conn(self, f):
         """
         连接close之后
@@ -187,3 +212,21 @@ class BlueprintCallBacksMixin(CallBacksMixin):
         """
         self.close_app_conn_func_list.append(f)
         return f
+
+    def repeat_app_timer(self, interval):
+        """
+        每隔一段时间执行(秒)
+        """
+        def inner_repeat_timer(func):
+            @functools.wraps(func)
+            def func_wrapper(*args, **kwargs):
+                # 每次也要再加入
+
+                result = safe_call(func, *args, **kwargs)
+                IOLoop.instance().add_timeout(time.time() + interval, func_wrapper)
+
+                return result
+
+            self.repeat_app_timer_func_list.append(func_wrapper)
+            return func_wrapper
+        return inner_repeat_timer
\ No newline at end of file
