@@ -1,7 +1,6 @@
 # -*- coding: utf-8 -*-
 
 import json
-from tornado import stack_context
 
 from . import constants
 from .request import Request
@@ -20,8 +19,6 @@ class Connection(object):
         self._clear_request_state()
 
         self.stream.set_close_callback(self._on_connection_close)
-        # 开始等待数据
-        self._read_message()
 
         # create_conn_func_list
         for func in self.app.create_conn_func_list:
@@ -31,6 +28,9 @@ class Connection(object):
             for func in bp.create_app_conn_func_list:
                 safe_call(func, self)
 
+        # 开始等待数据
+        self._read_message()
+
     def write(self, data, callback=None):
         """
         发送数据(实现是放到发送队列)
@@ -38,8 +38,16 @@ class Connection(object):
         if isinstance(data, dict):
             data = self.make_rsp(**data)
 
+        # before_app_response_func_list
+        for func in self.app.before_response_func_list:
+            safe_call(func, data)
+        # 执行blueprints
+        for name, bp in self.app.blueprints.items():
+            for func in bp.before_app_response_func_list:
+                safe_call(func, data)
+
         if not self.stream.closed():
-            self._write_callback = stack_context.wrap(callback)
+            self._write_callback = callback
             self.stream.write(data, self._on_write_complete)
 
     def finish(self):
@@ -68,7 +76,7 @@ class Connection(object):
     def set_close_callback(self, callback):
         """Sets a callback that will be run when the connection is closed.
         """
-        self._close_callback = stack_context.wrap(callback)
+        self._close_callback = callback
 
     def _on_connection_close(self):
         # 链接被关闭的回调
@@ -77,7 +85,7 @@ class Connection(object):
         if self._close_callback is not None:
             callback = self._close_callback
             self._close_callback = None
-            callback()
+            safe_call(callback)
 
         # 执行blueprints
         for name, bp in self.app.blueprints.items():
@@ -105,7 +113,7 @@ class Connection(object):
         if self._write_callback is not None:
             callback = self._write_callback
             self._write_callback = None
-            callback()
+            safe_call(callback)
         if self._request_finished and not self.stream.writing():
             self.close()
 
@@ -169,7 +177,7 @@ class Connection(object):
             view_func_result = view_func(request)
         except Exception, e:
             error = 'view_func raise exception. request: %s, view_func: %s, e: %s, traceback: %s' % (
-                request, view_func.__name__, e, __import__('traceback').format_exc())
+                request, view_func, e, __import__('traceback').format_exc())
             logger.error(error)
             view_func_exc = e
             request.write(dict(ret=constants.RET_SYSTEM,
